﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Windows.Browser;
using System.Xml.Linq;
using System.Linq;
using System.Diagnostics;
using System.Windows.Media.Imaging;
using MashupImageSearch.Model;
using System.Collections.Generic;
using System.ComponentModel;
using MashupImageSearch.Commands;
using System.Windows.Threading;

namespace MashupImageSearch.View
{
    /// <summary>
    /// ImageSearchView의 ViewModel 클래스
    /// </summary>
    public class ImageSearchViewModel : INotifyPropertyChanged
    {
        #region Properties

        /// <summary>
        /// 서비스타입들
        /// </summary>
        public enum ServiceTypes
        {
            LiveSearch = 0,
            Flickr = 1,
            Daum = 2,
            Naver = 3,
        }

        /// <summary>
        /// 마지막 서비스타입
        /// </summary>
        public ServiceTypes LastServiceType { get; set; }

        /// <summary>
        /// 포토 서비스
        /// </summary>
        private IPhotoService _photoService { get; set; }

        /// <summary>
        /// 새로운 검색이 가능한지 여부
        /// </summary>
        public bool CanSearch
        {
            get { return !IsSearching; }
        }

        #region SelectedPhoto

        private string _selectedPhotoUrl;
        /// <summary>
        /// 선택된 사진
        /// </summary>
        public string SelectedPhotoUrl
        {
            get { return _selectedPhotoUrl; }
            set
            {
                _selectedPhotoUrl = value;
                try
                {
                    SelectedPhoto = new BitmapImage(new Uri(_selectedPhotoUrl, UriKind.Absolute));
                }
                catch
                {
                    SelectedPhoto = null;
                }
                RaisePropertyChanged("SelectedPhotoUrl");
            }
        }

        private BitmapImage _selectedPhoto;
        /// <summary>
        /// 선택된 사진
        /// </summary>
        public BitmapImage SelectedPhoto
        {
            get { return _selectedPhoto; }
            set
            {
                if (_selectedPhoto != null)
                {
                    // 이벤트 해제
                    _selectedPhoto.DownloadProgress -= new EventHandler<DownloadProgressEventArgs>(_selectedPhoto_DownloadProgress);
                }

                _selectedPhoto = value;
                _selectedPhoto.DownloadProgress += new EventHandler<DownloadProgressEventArgs>(_selectedPhoto_DownloadProgress);

                RaisePropertyChanged("SelectedPhoto");
            }
        }

        #endregion SelectedPhoto

        #region DownloadProgress

        private double _downloadProgress;

        /// <summary>
        /// 다운로드 진행율
        /// </summary>
        public double DownloadProgress
        {
            get { return _downloadProgress; }
            set
            {
                _downloadProgress = value;
                RaisePropertyChanged("DownloadProgress");
            }
        }

        #endregion DownloadProgress

        #region ProgressVisible

        private Visibility _progressVisible = Visibility.Collapsed;

        /// <summary>
        /// 진행률 바를 보일지 여부
        /// </summary>
        public Visibility ProgressVisible
        {
            get { return _progressVisible; }
            set
            {
                _progressVisible = value;
                RaisePropertyChanged("ProgressVisible");
            }
        }

        #endregion ProgressVisible

        #region Message

        private string _message;
        public string Message
        {
            get { return _message; }
            set
            {
                _message = value;
                RaisePropertyChanged("Message");
            }
        }

        #endregion Message

        #region IsSearching

        private bool _isSearching = false;
        /// <summary>
        /// 현재 검색 중인지의 여부
        /// </summary>
        public bool IsSearching
        {
            get { return _isSearching; }
            set
            {
                _isSearching = value;
                RaisePropertyChanged("IsSearching", "CanSearch");
            }
        }

        #endregion IsSearching

        /// <summary>
        /// 한 페이지 당 검색 결과물 개수
        /// </summary>
        public readonly int PER_PAGE = 18;

        /// <summary>
        /// 현재 페이지
        /// </summary>
        public int CurrentPage { get; set; }

        /// <summary>
        /// 마지막으로 검색한 키워드
        /// </summary>
        public string LastKeyword { get; set; }

        /// <summary>
        /// 사진들
        /// </summary>
        public ObservableCollection<Photo> Photos { get; set; }

        #region SearchText

        private string _searchText;
        /// <summary>
        /// 검색어
        /// </summary>
        public string SearchText
        {
            get { return _searchText; }
            set
            {
                _searchText = value;
                RaisePropertyChanged("SearchText");
            }
        }
        #endregion SearchText

        #endregion Properties

        #region Constructors

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public ImageSearchViewModel()
        {
            InitProperties();
            SearchCommands.Search.Executed += new EventHandler<HugeFlow.CommandPattern.ExecutedEventArgs>(Search_Executed);
            SearchCommands.Navigate.Executed += new EventHandler<HugeFlow.CommandPattern.ExecutedEventArgs>(Navigate_Executed);
            SearchCommands.ShowLargeImage.Executed += new EventHandler<HugeFlow.CommandPattern.ExecutedEventArgs>(ShowLargeImage_Executed);
            SearchCommands.ChangeService.Executed += new EventHandler<HugeFlow.CommandPattern.ExecutedEventArgs>(ChangeService_Executed);
            SearchCommands.NavigateOriginalImage.Executed += new EventHandler<HugeFlow.CommandPattern.ExecutedEventArgs>(NavigateOriginalImage_Executed);
        }

        #endregion Constructors

        #region Command EventHandlers

        void _selectedPhoto_DownloadProgress(object sender, DownloadProgressEventArgs e)
        {
            try
            {
                if (e.Progress >= 100)
                {
                    DownloadProgress = 0.0;
                    Message = "";
                    ProgressVisible = Visibility.Collapsed;
                }
                else
                {
                    DownloadProgress = e.Progress;
                    Message = string.Format("{0}%", e.Progress);
                    ProgressVisible = Visibility.Visible;
                }
            }
            catch (Exception ex) // 가끔 e.Progress에서 Overflow Exception이 발생함
            {
                Debug.WriteLine(ex.Message);
            }
        }

        void NavigateOriginalImage_Executed(object sender, HugeFlow.CommandPattern.ExecutedEventArgs e)
        {
            string link = (string)e.Parameter;

            if (string.IsNullOrEmpty(link) == true)
                return;

            HtmlPage.Window.Navigate(new Uri(link), "_blank");
        }

        void ChangeService_Executed(object sender, HugeFlow.CommandPattern.ExecutedEventArgs e)
        {
            ServiceTypes serviceType = (ServiceTypes)e.Parameter;

            switch (serviceType)
            {
                default:
                case ServiceTypes.LiveSearch:
                    _photoService = new LivePhotoService();
                    break;
                case ServiceTypes.Flickr:
                    _photoService = new FlickrPhotoService();
                    break;
                case ServiceTypes.Daum:
                    _photoService = new DaumPhotoService();
                    break;
                case ServiceTypes.Naver:
                    _photoService = new NaverPhotoService();
                    break;
            }

            if (_photoService != null)
            {
                _photoService.SearchImageCompleted += new PhotoSearchEventHandler(_photoService_SearchImageCompleted);
            }

            InitSearch();
        }

        void ShowLargeImage_Executed(object sender, HugeFlow.CommandPattern.ExecutedEventArgs e)
        {
            Message = "";
            SelectedPhotoUrl = (string)e.Parameter;
        }

        void Navigate_Executed(object sender, HugeFlow.CommandPattern.ExecutedEventArgs e)
        {
            string link = (string)e.Parameter;
            if (string.IsNullOrEmpty(link) == true)
                return;

            HtmlPage.Window.Navigate(new Uri(link, UriKind.Absolute), "_blank");
        }

        void Search_Executed(object sender, HugeFlow.CommandPattern.ExecutedEventArgs e)
        {
            Search((string)e.Parameter);
        }

        #endregion Command EventHandlers

        #region EventHandlers

        void _photoService_SearchImageCompleted(string keyword, List<Photo> photos)
        {
            // 새로운 단어로 검색한 결과이면
            if (LastKeyword != keyword)
            {
                // 이전 사진을 모두 삭제
                Photos.Clear();
            }

            // 반환 값이 null이면 아무 할 일이 없다.
            if (photos == null)
                return;

            // 사진을 추가
            foreach (Photo photo in photos)
            {
                Photos.Insert(0, photo);
            }

            // 최근 검색어 백업
            LastKeyword = keyword;

            // 검색이 끝났음을 설정
            IsSearching = false;
        }

        void _timerMock_Tick(object sender, EventArgs e)
        {
            (sender as DispatcherTimer).Stop();
            _photoService.SearchImageAsync("테스트", 1, 30);
            SelectedPhotoUrl = "http://imgnews.naver.com/image/011/2009/01/16/psi1204200901161148270.jpg";
        }

        #endregion EventHandlers

        #region Methods

        /// <summary>
        /// 검색을 초기화합니다.
        /// </summary>
        private void InitSearch()
        {
            CurrentPage = 0;
            LastKeyword = "";
        }

        /// <summary>
        /// 프로퍼티들을 초기화합니다.
        /// </summary>
        private void InitProperties()
        {
            if (App.IsBlend == true) // 블렌드에서 작동 중인 경우
            {
                // Mockup 클래스를 생성함
                _photoService = new MockPhotoService();
                _photoService.SearchImageCompleted += new PhotoSearchEventHandler(_photoService_SearchImageCompleted);

                // 타이머에 의해서 검색이 자동으로 수행되도록 함
                DispatcherTimer _timerMock = new DispatcherTimer();
                _timerMock.Tick += new EventHandler(_timerMock_Tick);
                _timerMock.Start();
            }
            else
            {
                _photoService = new LivePhotoService();
                _photoService.SearchImageCompleted += new PhotoSearchEventHandler(_photoService_SearchImageCompleted);
            }

            // 프로퍼티 초기화
            Photos = new ObservableCollection<Photo>();
            CurrentPage = 0;
            LastKeyword = "";
            LastServiceType = ServiceTypes.LiveSearch;
        }

        /// <summary>
        /// 검색을 합니다.
        /// </summary>
        /// <param name="keyword"></param>
        public void Search(string keyword)
        {
            // 이전 검색어와 같은 검색어라면 페이지만 증가합니다.
            if (LastKeyword == keyword)
            {
                CurrentPage++;
            }
            else // 새로운 검색어라면 첫 페이지부터 다시 검색
            {
                CurrentPage = 1;
            }

            IsSearching = true;

            // 검색을 비동기 호출
            _photoService.SearchImageAsync(keyword, CurrentPage, PER_PAGE);
        }

        #endregion Methods

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void RaisePropertyChanged(params string[] parameters)
        {
            if (PropertyChanged == null)
                return;

            foreach (string param in parameters)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(param));
            }
        }
        #endregion
    }
}
